@@ -1,6 +1,3 @@
-// Based on https://gist.github.com/diwic/5c20a283ca3a03752e1a27b0f3ebfa30
-// See https://old.reddit.com/r/rust/comments/4xneq5/the_calendar_example_challenge_ii_why_eddyb_all/
-
 use anyhow::Context as AnyhowContext;
 use anyhow::{anyhow, Result};
 use std::fmt;
@@ -43,12 +40,14 @@ pub struct ProjectsState {
     pub rows: Vec<ProjectDetails>,
 }
 
+#[derive(Debug, Clone, Default)]
 pub struct ProjectDetails {
     name: Project,
     remaining: usize,
     avg_age: String,
     complete: String,
 }
+
 impl ProjectsState {
     pub(crate) fn new() -> Self {
         Self {
@@ -66,6 +65,7 @@ impl ProjectsState {
             rows: vec![],
         }
     }
+
     fn pattern_by_marked(app: &mut TaskwarriorTui) -> String {
         let mut project_pattern = String::new();
         if !app.projects.marked.is_empty() {
@@ -74,16 +74,17 @@ impl ProjectsState {
                 if input.as_str() == "(none)" {
                     input = " ".to_string();
                 }
-                if idx > 0 {
-                    project_pattern = format!("{} or project:{}", project_pattern, input);
-                } else {
+                if idx == 0 {
                     project_pattern = format!("\'(project:{}", input);
+                } else {
+                    project_pattern = format!("{} or project:{}", project_pattern, input);
                 }
             }
             project_pattern = format!("{})\'", project_pattern);
         }
         project_pattern
     }
+
     pub fn toggle_mark(&mut self) {
         if !self.list.is_empty() {
             let selected = self.current_selection;
@@ -92,6 +93,7 @@ impl ProjectsState {
             }
         }
     }
+
     pub fn simplified_view(&mut self) -> (Vec<Vec<String>>, Vec<String>) {
         let rows = self
             .rows
@@ -118,62 +120,54 @@ impl ProjectsState {
             .context("Unable to run `task summary`")
             .unwrap();
         let data = String::from_utf8_lossy(&output.stdout);
-        let contains_avg_age = data
-            .split('\n')
-            .into_iter()
-            .skip(1)
-            .collect::<Vec<&str>>()
-            .first()
-            .unwrap()
-            .contains("Avg age");
-        for line in data.split('\n').into_iter().skip(3) {
-            if line.is_empty() {
-                break;
-            }
-            let mut row: Vec<String> = line
-                .split(' ')
-                .map(str::trim)
-                .map(str::trim_start)
-                .filter(|x| !x.is_empty())
-                .filter(|x| !x.chars().all(|c| c == '='))
-                .map(ToString::to_string)
-                .collect();
-            assert!(row.len() >= 3);
-            let (complete, avg_age, remaining, name) = if contains_avg_age {
-                (
-                    row.pop().unwrap().parse().unwrap(),
-                    row.pop().unwrap().parse().unwrap(),
-                    row.pop().unwrap().parse().unwrap(),
-                    row.join(" "),
-                )
-            } else {
-                (
-                    row.pop().unwrap().parse().unwrap(),
-                    "0s".to_string(),
-                    row.pop().unwrap().parse().unwrap(),
-                    row.join(" "),
-                )
-            };
 
-            self.rows.push(ProjectDetails {
-                name,
-                remaining,
-                avg_age,
-                complete,
-            });
+        let lines = data.split('\n').into_iter().skip(1).collect::<Vec<&str>>();
+
+        let header = lines.first().unwrap();
+
+        let contains_avg_age = header.contains("Avg age");
+
+        if contains_avg_age {
+            let name_index = header.find("Remaining").unwrap();
+            let remaining_index = header.find("Remaining").unwrap() + "Remaining".len();
+            let average_age_index = header.find("Avg age").unwrap() + "Avg age".len();
+            let complete_index = header.find("Complete").unwrap() + "Complete".len();
+
+            for line in lines.into_iter().skip(2) {
+                if line.is_empty() {
+                    break;
+                }
+
+                let line = line.to_string();
+                let name = line[0..name_index].trim().to_string();
+                let remaining = line[name_index..remaining_index].trim().parse().unwrap();
+                let avg_age = line[remaining_index..average_age_index].trim().to_string();
+                let complete = line[average_age_index..complete_index].trim().to_string();
+
+                self.rows.push(ProjectDetails {
+                    name,
+                    remaining,
+                    avg_age,
+                    complete,
+                });
+            }
         }
+
         self.list = self.rows.iter().map(|x| x.name.clone()).collect_vec();
         Ok(())
     }
+
     fn update_table_state(&mut self) {
         self.table_state.select(Some(self.current_selection));
         if self.marked.is_empty() {
             self.table_state.single_selection();
-        }
-        self.table_state.clear();
-        for project in &self.marked {
-            let index = self.list.iter().position(|x| x == project);
-            self.table_state.mark(index);
+        } else {
+            self.table_state.multiple_selection();
+            self.table_state.clear();
+            for project in &self.marked {
+                let index = self.list.iter().position(|x| x == project);
+                self.table_state.mark(index);
+            }
         }
     }
 }
@@ -218,7 +212,7 @@ fn update_task_filter_by_selection(app: &mut TaskwarriorTui) -> Result<()> {
     app.projects.toggle_mark();
     let new_project_pattern = ProjectsState::pattern_by_marked(app);
     let current_filter = app.filter.as_str();
-    app.filter_history_context.add(current_filter);
+    app.filter_history.add(current_filter);
 
     let mut filter = current_filter.replace(&last_project_pattern, "");
     filter = format!("{}{}", filter, new_project_pattern);
@@ -226,3 +220,18 @@ fn update_task_filter_by_selection(app: &mut TaskwarriorTui) -> Result<()> {
     app.update(true)?;
     Ok(())
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_project_summary() {
+        let mut app = TaskwarriorTui::new("next").unwrap();
+
+        app.update(true).unwrap();
+
+        dbg!(&app.projects.rows);
+        dbg!(&app.projects.list);
+    }
+}
